@@ -3,7 +3,10 @@
 
 import appdirs
 import argcomplete, argparse
+import base64
 from cryptography.fernet import Fernet
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 import getpass
 import json
 import keyring
@@ -25,17 +28,20 @@ def err(rc, msg):
 
 def warn(msg):
   print(f"{sys.argv[0]}: {msg}", file=sys.stderr)
-def store_ladok_session(ls):
+def store_ladok_session(ls, credentials):
   if not os.path.isdir(dirs.user_cache_dir):
     os.mkdir(dirs.user_cache_dir)
 
   file_path = dirs.user_cache_dir + "/LadokSession"
 
   pickled_ls = pickle.dumps(ls)
-  key = keyring.get_password("ladok3", "cache_key")
-  if not key:
-    key = Fernet.generate_key()
-  keyring.set_password("ladok3", "cache_key", key)
+  kdf = PBKDF2HMAC(
+    algorithm=hashes.SHA256(),
+    length=32,
+    salt=credentials[0].encode("utf-8"),
+    iterations=100000
+  )
+  key = base64.urlsafe_b64encode(kdf.derive(credentials[1].encode("utf-8")))
 
   fernet_protocol = Fernet(key)
   encrypted_ls = fernet_protocol.encrypt(pickled_ls)
@@ -43,13 +49,19 @@ def store_ladok_session(ls):
   with open(file_path, "wb") as file:
     file.write(encrypted_ls)
   
-def restore_ladok_session():
+def restore_ladok_session(credentials):
   file_path = dirs.user_cache_dir + "/LadokSession"
 
   if os.path.isfile(file_path):
     with open(file_path, "rb") as file:
       encrypted_ls = file.read()
-      key = keyring.get_password("ladok3", "cache_key")
+      kdf = PBKDF2HMAC(
+        algorithm=hashes.SHA256(),
+        length=32,
+        salt=credentials[0].encode("utf-8"),
+        iterations=100000
+      )
+      key = base64.urlsafe_b64encode(kdf.derive(credentials[1].encode("utf-8")))
 
       fernet_protocol = Fernet(key)
       pickled_ls = fernet_protocol.decrypt(encrypted_ls)
@@ -58,7 +70,7 @@ def restore_ladok_session():
   return None
 def update_credentials_in_keyring(ls, args):
   user = input("LADOK username: ")
-  passwd = getpass.getpass("LADOK password: ")
+  passwd = getpass.getpass("LADOK password: [input is hidden]")
 
   keyring.set_password("ladok3", "username", user)
   keyring.set_password("ladok3", "password", passwd)
@@ -155,13 +167,13 @@ def main():
   ladok3.report.add_command_options(subp)
   argcomplete.autocomplete(argp)
   args = argp.parse_args()
-  ls = restore_ladok_session()
+  LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
+  ls = restore_ladok_session([LADOK_USER, LADOK_PASS])
   if not ls:
-    LADOK_USER, LADOK_PASS = load_credentials(args.config_file)
     ls = ladok3.LadokSessionKTH(LADOK_USER, LADOK_PASS)
   if "func" in args:
     args.func(ls, args)
-  store_ladok_session(ls)
+  store_ladok_session(ls, [LADOK_USER, LADOK_PASS])
 
 if __name__ == "__main__":
   try:
